<?php

namespace Drupal\sl_stats\Commands;

use Drupal\Core\Lock\LockBackendInterface;
use Drupal\sl_stats\SLStatsComputer;
use Drush\Commands\DrushCommands;

/**
 * A drush command file.
 *
 * @package Drupal\sl_stats\Commands
 */
class SLStatsCommands extends DrushCommands {

  /**
   * The SL stats computer.
   *
   * @var \Drupal\sl_stats\SLStatsComputer
   */
  protected $computer;

  public function __construct(SLStatsComputer $sl_stats_computer, LockBackendInterface $lock) {
    $this->computer = $sl_stats_computer;
    $this->lock = $lock;
  }

  /**
   * Drush command that resets stats for players and seasons.
   *
   * @command sl_stats:reset
   * @usage sl_stats:reset
   */
  public function reset() {
    $this->computer->sl_stats_reset();
  }

  /**
   * Drush command that generates stats for players and seasons.
   *
   * @param string $id
   *   The player id to compute.
   * @command sl_stats:compute
   * @usage sl_stats:compute
   */
  public function computes($id) {
    if ($this->lock->acquire('sl_stats_autostats')) {
      $this->computer->compute($id);
      $this->lock->release('sl_stats_autostats');
    }
  }

}
