<?php

namespace Drupal\sl_stats;

use Drupal\Core\Entity\ContentEntityInterface;

abstract class SLStatsComputerFull extends SLStatsComputerBase {

  protected $teamStatsType;
  protected $matchesView;
  protected $matchesDisplayId;
  protected $momentsView;
  protected $momentsDisplayId;

  protected $view_matches;
  protected $competitions;

  public function getTotals() {

  }

  function preCompute($player, $team) {
    if (empty($this->matchesView)) {
      return;
    }
    // computes all matches
    $this->view_matches = $this->getViewsResults($this->matchesView, $this->matchesDisplayId, [
      $player->id(),
      $team->id()
    ]);

    if (!empty($this->view_matches)) {
      $this->total_team['matches'] = count($this->view_matches) ?? 0;
    }
    else {
      $this->total_team['matches'] = 0;
    }

    if (!empty($this->momentsView)) {
      // computes all moments
      $this->view_moments = $this->getViewsResults($this->momentsView, $this->momentsDisplayId, [
        $player->id(),
        $team->id()
      ]);
    }
  }

  protected function analyzeMatch($match, $player, $team) {

  }

  public function compute($player, $team) {

    $this->competitions = $this->friendlies = $this->total_team = array();

    // computes all matches done in this team
    $this->preCompute($player, $team);

    // iterate on all match moments
    if (!empty($this->view_moments)) {
      foreach ($this->view_moments as $match) {
        $match = (object) $match;

        // Non official matches.
        if ($match->field_sl_stats_disabled === '1') {
          if (empty($this->friendlies['unofficial_moments'][$match->type])) {
            $this->friendlies['unofficial_moments'][$match->type] = 1;
          } else {
            $this->friendlies['unofficial_moments'][$match->type]++;
          }

          if (!isset($this->total_team['unofficial_moments'][$match->type])) {
            $this->total_team['unofficial_moments'][$match->type] = 1;
          } else {
            $this->total_team['unofficial_moments'][$match->type]++;
          }

        }
        else {
          if (empty($this->competitions[$match->field_sl_competition[0]]) || empty($this->competitions[$match->field_sl_competition[0]]['moments'][$match->type])) {
            $this->competitions[$match->field_sl_competition[0]]['moments'][$match->type] = 1;
          } else {
            $this->competitions[$match->field_sl_competition[0]]['moments'][$match->type]++;
          }

          if (!isset($this->total_team['moments'][$match->type])) {
            $this->total_team['moments'][$match->type] = 1;
          } else {
            $this->total_team['moments'][$match->type]++;
          }
        }
      }
    }

    $total_matches = $friendly_matches = 0;
    $this->friendlies['matches'] = $this->friendlies['minutes'] = 0;
    // Iterate on all matches.
    if (!empty($this->view_matches)) {
      foreach ($this->view_matches as $match) {
        $match = (object) $match;

        $this->analyzeMatch($match, $player, $team);

        if ($match->field_sl_stats_disabled === '1') {
          if (!isset($this->friendlies['unofficial_matches'])) {
            $this->friendlies['unofficial_matches'] = 0;
          }
          $this->friendlies['unofficial_matches']++;
          $friendly_matches++;
        }
        else {
          if (isset($match->field_sl_roster_in) && isset($match->field_sl_roster_in)) {
            $in = $match->field_sl_roster_in;
            $out = $match->field_sl_roster_out;

            // find match time
            if (!is_numeric($in) || !is_numeric($out)) {
              $time = 0;
            }
            else {
              $time = $out - $in;
            }

            if ($time < 0) {
              $time = 0;
            }

            if (!isset($this->total_team['minutes'])) {
              $this->total_team['minutes'] = 0;
            }
            $this->total_team['minutes'] += $time;

            if (!isset($this->competitions[$match->field_sl_competition[0]]['minutes'])) {
              $this->competitions[$match->field_sl_competition[0]]['minutes'] = 0;
            }

            $this->competitions[$match->field_sl_competition[0]]['minutes'] += $time;
          }

          $total_matches++;

          if (!isset($this->competitions[$match->field_sl_competition[0]]['matches'])) {
            $this->competitions[$match->field_sl_competition[0]]['matches'] = 0;
          }
          $this->competitions[$match->field_sl_competition[0]]['matches']++;
        }
      }
      $this->total_team['matches'] = $total_matches;
      $this->total_team['unofficial_matches'] = $friendly_matches;
    }

    // depends on the stats entity type
    $entity = $this->stats_manager->create(array('type' => $this->teamStatsType));
    $this->createEntity($entity, $this->total_team, $player->id(), $team->id());
    \Drupal::moduleHandler()->alter('sl_stats_person', $entity, $player);
    if ($entity instanceof ContentEntityInterface) {
      $entity->save();
    }

    if (!isset($player->total_stats['matches'])) {
      $player->total_stats['matches'] = 0;
    }
    $player->total_stats['matches'] += $this->total_team['matches'];

    if (!isset($player->total_stats['goals'])) {
      $player->total_stats['goals'] = 0;
    }
    $player->total_stats['goals'] += empty($this->total_team['moments']['sl_match_moments_goal']) ? 0 : $this->total_team['moments']['sl_match_moments_goal'];
    $this->postCompute($player, $team);
  }

  function postCompute($player, $team) {
    if (!empty($this->competitions)) {
      foreach ($this->competitions as $key => $value) {
        if (is_numeric($key)) {
          $entity_c = $this->stats_manager->create(array('type' => $this->teamStatsType));
          $this->createEntity($entity_c, $value, $player->id(), $team->id(), $key);
          \Drupal::moduleHandler()->alter('sl_stats_person', $entity_c, $node);
          $entity_c->save();
        }
      }
    }

    if (!empty($this->friendlies) && !empty($this->teamStatsTypeNonOfficial)) {
      $entity_c = $this->stats_manager->create(array('type' => $this->teamStatsTypeNonOfficial));
      $this->createEntity($entity_c, $this->friendlies, $player->id(), $team->id());
      \Drupal::moduleHandler()->alter('sl_stats_person', $entity_c, $node);
      $entity_c->save();
    }
  }

}
